@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -39,13 +34,17 @@ $fields = [
 	'outer' =>			[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null],
 	'onlyHeight' =>		[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null],
 	'legend' =>			[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null],
-	'widget_view' =>	[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null]
+	'widget_view' =>	[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null],
+	'resolve_macros' =>	[T_ZBX_INT,			O_OPT, null,	IN('0,1'),	null]
 ];
 if (!check_fields($fields)) {
+	session_write_close();
 	exit();
 }
 validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
 
+$resolve_macros = (bool) getRequest('resolve_macros', 0);
+
 /*
  * Permissions
  */
@@ -53,8 +52,8 @@ $dbGraph = API::Graph()->get([
 	'output' => API_OUTPUT_EXTEND,
 	'selectGraphItems' => API_OUTPUT_EXTEND,
 	'selectHosts' => ['hostid', 'name', 'host'],
-	'selectItems' => ['itemid', 'type', 'master_itemid', 'name', 'delay', 'units', 'hostid', 'history', 'trends',
-		'value_type', 'key_'
+	'selectItems' => ['itemid', 'type', 'master_itemid', $resolve_macros ? 'name_resolved' : 'name', 'delay', 'units',
+		'hostid', 'history', 'trends', 'value_type', 'key_'
 	],
 	'graphids' => $_REQUEST['graphid']
 ]);
@@ -62,27 +61,13 @@ $dbGraph = API::Graph()->get([
 if (!$dbGraph) {
 	access_deny();
 }
-else {
-	$dbGraph = reset($dbGraph);
+
+$dbGraph = reset($dbGraph);
+
+if ($resolve_macros) {
+	$dbGraph['items'] = CArrayHelper::renameObjectsKeys($dbGraph['items'], ['name_resolved' => 'name']);
 }
 
-$hosts = zbx_toHash($dbGraph['hosts'], 'hostid');
-$items = zbx_toHash($dbGraph['items'], 'itemid');
-
-$db_items = API::Item()->get([
-	'output' => [],
-	'selectPreprocessing' => ['type', 'params'],
-	'itemids' => array_keys($items),
-	'webitems' => true,
-	'nopermissions' => true,
-	'preservekeys' => true
-]);
-
-foreach ($items as &$item) {
-	$item['preprocessing'] = $db_items[$item['itemid']]['preprocessing'];
-}
-unset($item);
-
 /*
  * Display
  */
@@ -106,6 +91,9 @@ CArrayHelper::sort($dbGraph['gitems'], [
 	['field' => 'itemid', 'order' => ZBX_SORT_DOWN]
 ]);
 
+$hosts = zbx_toHash($dbGraph['hosts'], 'hostid');
+$items = zbx_toHash($dbGraph['items'], 'itemid');
+
 foreach ($dbGraph['gitems'] as $graph_item) {
 	$item = $items[$graph_item['itemid']];
 	$host = $hosts[$item['hostid']];
@@ -155,8 +143,29 @@ $graph->setYMinAxisType($dbGraph['ymin_type']);
 $graph->setYMaxAxisType($dbGraph['ymax_type']);
 $graph->setYAxisMin($dbGraph['yaxismin']);
 $graph->setYAxisMax($dbGraph['yaxismax']);
-$graph->setYMinItemId($dbGraph['ymin_itemid']);
-$graph->setYMaxItemId($dbGraph['ymax_itemid']);
+
+$yaxis_items = array_intersect_key($dbGraph, array_filter([
+	'ymin_itemid' => $dbGraph['ymin_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE && $dbGraph['ymin_itemid'] != 0,
+	'ymax_itemid' => $dbGraph['ymax_type'] == GRAPH_YAXIS_TYPE_ITEM_VALUE && $dbGraph['ymax_itemid'] != 0
+]));
+
+if ($yaxis_items) {
+	$db_items = API::Item()->get([
+		'itemids' => array_values($yaxis_items),
+		'filter' => ['value_type' => [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64]],
+		'webitems' => true,
+		'preservekeys' => true
+	]);
+
+	if (array_key_exists('ymin_itemid', $yaxis_items) && array_key_exists($yaxis_items['ymin_itemid'], $db_items)) {
+		$graph->setYMinItemId($yaxis_items['ymin_itemid']);
+	}
+
+	if (array_key_exists('ymax_itemid', $yaxis_items) && array_key_exists($yaxis_items['ymax_itemid'], $db_items)) {
+		$graph->setYMaxItemId($yaxis_items['ymax_itemid']);
+	}
+}
+
 $graph->setLeftPercentage($dbGraph['percent_left']);
 $graph->setRightPercentage($dbGraph['percent_right']);
 
@@ -164,12 +173,12 @@ if (hasRequest('outer')) {
 	$graph->setOuter(getRequest('outer'));
 }
 
-$min_dimentions = $graph->getMinDimensions();
-if ($min_dimentions['width'] > $graph->getWidth()) {
-	$graph->setWidth($min_dimentions['width']);
+$min_dimensions = $graph->getMinDimensions();
+if ($min_dimensions['width'] > $graph->getWidth()) {
+	$graph->setWidth($min_dimensions['width']);
 }
-if ($min_dimentions['height'] > $graph->getHeight()) {
-	$graph->setHeight($min_dimentions['height']);
+if ($min_dimensions['height'] > $graph->getHeight()) {
+	$graph->setHeight($min_dimensions['height']);
 }
 
 if (getRequest('onlyHeight', '0') === '1') {
